package ruleflowtest.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.compiler.DroolsParserException;
import org.drools.compiler.PackageBuilder;

import ruleflowtest.fact.FactA;
import ruleflowtest.fact.FactB;

public class RuleflowTestMain {

    private final PackageBuilder packageBuilder = new PackageBuilder();

    private final RuleBase ruleBase = RuleBaseFactory.newRuleBase();

    private WorkingMemory workingMemory;

    public static void main(String[] args) throws IOException, DroolsParserException, Exception {

        final RuleflowTestMain main = new RuleflowTestMain();

        main.buildPackage();
        main.createWorkingMemory();
        main.insertOneFact();
//        main.insertBothFacts();
        main.runRules();
    }

    private void buildPackage() throws IOException, DroolsParserException, Exception {

        final InputStream rfmInputStream = getClass().getResourceAsStream("/ruleflowtest/drools/ruleflowTest.rfm");
        final InputStream drlInputStream = getClass().getResourceAsStream("/ruleflowtest/drools/ruleflowTest.drl");
        final InputStreamReader rfmInputStreamReader = new InputStreamReader(rfmInputStream);
        final InputStreamReader drlInputStreamReader = new InputStreamReader(drlInputStream);

        packageBuilder.addRuleFlow(rfmInputStreamReader);
        packageBuilder.addPackageFromDrl(drlInputStreamReader);
        ruleBase.addPackage(packageBuilder.getPackage());
    }

    private void createWorkingMemory() {
        workingMemory = ruleBase.newStatefulSession();
    }

    private void insertBothFacts() {
        workingMemory.insert(new FactA());
        workingMemory.insert(new FactB());
    }

    private void insertOneFact() {
        workingMemory.insert(new FactA());
    }

    private void runRules() {
        workingMemory.startProcess("ruleflowTest");
        workingMemory.fireAllRules();
    }

}
